@@ -21,7 +21,7 @@ import zim.errors
 
 from zim.fs import adapt_from_oldfs
 from zim.newfs import SEP, is_abs_filepath, FilePath, LocalFile
-from zim.parsing import is_uri_re, is_win32_path_re
+from zim.parse.links import is_uri_re, is_win32_path_re
 
 
 logger = logging.getLogger('zim.applications')
@@ -301,7 +301,7 @@ class Application(object):
 
 			p = subprocess.Popen(argv,
 				cwd=cwd,
-				stdout=open(os.devnull, 'w'),
+				stdout=subprocess.DEVNULL,
 				stderr=subprocess.PIPE,
 				startupinfo=info,
 				bufsize=4096,
@@ -311,7 +311,7 @@ class Application(object):
 			try:
 				p = subprocess.Popen(argv,
 					cwd=cwd,
-					stdout=open(os.devnull, 'w'),
+					stdout=subprocess.DEVNULL,
 					stderr=subprocess.PIPE,
 					bufsize=4096,
 					close_fds=True
@@ -320,7 +320,7 @@ class Application(object):
 				if _CAN_CALL_FLATPAK_HOST_COMMAND:
 					p = subprocess.Popen(_FLATPAK_HOSTCOMMAND_PREFIX + argv,
 						cwd=cwd,
-						stdout=open(os.devnull, 'w'),
+						stdout=subprocess.DEVNULL,
 						stderr=subprocess.PIPE,
 						bufsize=4096,
 						close_fds=True
@@ -353,8 +353,13 @@ class Application(object):
 		if TEST_MODE:
 			return TEST_MODE_RUN_CB(argv)
 
+		startupinfo = None
+		if os.name == 'nt':
+			startupinfo = subprocess.STARTUPINFO()
+			startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+			startupinfo.wShowWindow = subprocess.SW_HIDE
 		try:
-			p = subprocess.Popen(argv, cwd=cwd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+			p = subprocess.Popen(argv, cwd=cwd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, startupinfo=startupinfo)
 		except OSError:
 			if _CAN_CALL_FLATPAK_HOST_COMMAND:
 				p = subprocess.Popen(_FLATPAK_HOSTCOMMAND_PREFIX + argv, cwd=cwd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
@@ -371,7 +376,7 @@ class Application(object):
 			#~ raise ApplicationError(argv[0], argv[1:], p.returncode, stderr)
 		#~ elif stderr:
 		if stderr:
-			logger.warn(_decode(stderr))
+			logger.warning(_decode(stderr))
 			# TODO: allow user to get this error as well - e.g. for logging image generator cmd
 
 		text = _decode(stdout).replace('\r\n', '\n').splitlines(keepends=True)
@@ -411,6 +416,11 @@ class Application(object):
 			TEST_MODE_RUN_CB(argv)
 			return None
 
+		# https://github.com/zim-desktop-wiki/zim-desktop-wiki/issues/1697
+		def _callback_wrapper(pid, *args):
+			GLib.spawn_close_pid(pid)
+			callback(*args)
+
 		try:
 			try:
 				pid, stdin, stdout, stderr = \
@@ -432,10 +442,10 @@ class Application(object):
 				# child watch does implicit reaping -> no zombies
 				if data is None:
 					GObject.child_watch_add(pid,
-						lambda pid, status: callback(status))
+						lambda _, status: _callback_wrapper(pid, status))
 				else:
 					GObject.child_watch_add(pid,
-						lambda pid, status, data: callback(status, data), data)
+						lambda _, status, data: _callback_wrapper(pid, status, data), data)
 			return pid
 
 
