@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2012,2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -13,18 +12,18 @@
 # - add \ before line ends to match line breaks from user
 
 import re
+from zim.parse.encode import escape_string
 
 from zim.formats import *
-from zim.parsing import url_re
+from zim.parse.links import old_url_link_re
 from zim.formats.plain import Dumper as TextDumper
 
 
 info = {
 	'name': 'markdown',
 	'desc': 'Markdown Text (pandoc)',
-	'mimetype': 'text/x-markdown',
-	'extension': 'markdown',
-		# No official file extension, but this is often used
+	'mimetype': 'text/markdown',
+	'extension': 'md', # Most common extention used on github.
 	'native': False,
 	'import': False,
 	'export': True,
@@ -38,17 +37,18 @@ class Dumper(TextDumper):
 	# are different
 
 	BULLETS = {
-		UNCHECKED_BOX: u'* \u2610', # ☐
-		XCHECKED_BOX: u'* \u2612', # ☒
-		CHECKED_BOX: u'* \u2611', # ☑
-		MIGRATED_BOX: u'* \u25B7', # ▷
-		BULLET: u'*',
+		UNCHECKED_BOX: '* \u2610',
+		XCHECKED_BOX: '* \u2612',
+		CHECKED_BOX: '* \u2611',
+		MIGRATED_BOX: '* \u25B7',
+		TRANSMIGRATED_BOX: '* \u25C1',
+		BULLET: '*',
 	}
 
 	TAGS = {
 		EMPHASIS: ('*', '*'),
 		STRONG: ('**', '**'),
-		MARK: ('__', '__'), # OPEN ISSUE: not availalbe in pandoc
+		MARK: ('__', '__'), # OPEN ISSUE: not available in pandoc
 		STRIKE: ('~~', '~~'),
 		VERBATIM: ("``", "``"),
 		TAG: ('', ''), # No additional annotation (apart from the visible @)
@@ -61,8 +61,13 @@ class Dumper(TextDumper):
 		return TextDumper.dump(self, tree)
 
 	def dump_indent(self, tag, attrib, strings):
-		# OPEN ISSUE: no indent for para
-		return strings
+		# Prefix lines with one or more tabs
+		if attrib and 'indent' in attrib:
+			prefix = '> ' * int(attrib['indent'])
+			return self.prefix_lines(prefix, strings)
+			# TODO enforces we always end such a block with \n unless partial
+		else:
+			return strings
 
 	dump_p = dump_indent
 	dump_div = dump_indent
@@ -73,7 +78,7 @@ class Dumper(TextDumper):
 			del attrib['indent']
 		strings = TextDumper.dump_list(self, tag, attrib, strings)
 
-		if self.context[-1].tag in (BULLETLIST, NUMBEREDLIST):
+		if self.context[-1].tag == LISTITEM:
 			# sub-list
 			return strings
 		else:
@@ -85,25 +90,58 @@ class Dumper(TextDumper):
 	dump_ul = dump_list
 	dump_ol = dump_list
 
+	def dump_li(self, tag, attrib, strings):
+		# Markdown does not support letters as list bullets - convert to number
+		assert self.context[-1].tag in (BULLETLIST, NUMBEREDLIST), 'Do not support raw pageview output here'
+		if self.context[-1].tag == NUMBEREDLIST \
+			and not self.context[-1].attrib.get('_iter'):
+				# First item on this level
+				iter = self.context[-1].attrib.get('start', '1')
+				self.context[-1].attrib['_iter'] = convert_list_iter_letter_to_number(iter)
+		return TextDumper.dump_li(self, tag, attrib, strings)
+
 	def dump_pre(self, tag, attrib, strings):
 		# OPEN ISSUE: no indent for verbatim blocks
 		return self.prefix_lines('\t', strings)
 
+	def dump_anchor(self, tag, attrib, strings=None):
+		# Syntax of the pandoc header attributes extension
+		return ('{#%s}' % attrib['name'],)
+
 	def dump_link(self, tag, attrib, strings=None):
 		assert 'href' in attrib, \
 			'BUG: link misses href: %s "%s"' % (attrib, strings)
 		href = self.linker.link(attrib['href'])
-		text = u''.join(strings) or href
-		if href == text and url_re.match(href):
+		text = ''.join(strings) or href
+		if href == text and old_url_link_re.match(href):
 			return ['<', href, '>']
 		else:
 			return ['[%s](%s)' % (text, href)]
 
 	def dump_img(self, tag, attrib, strings=None):
-		# OPEN ISSUE: image properties used in zim not supported in pandoc
 		src = self.linker.img(attrib['src'])
 		text = attrib.get('alt', '')
-		return ['![%s](%s)' % (text, src)]
+
+		# Handle image dimensions and ids
+		# Pandoc support setting dimensions like this:
+		# ![Alt text](href){ width=500px height=20px }
+		dimensions = filter(lambda i: i[0] in ['width', 'height'], attrib.items())
+		properties = ["%s=%spx" % (k, v) for k, v in dimensions]
+
+		if 'id' in attrib:
+			properties.append('#' + attrib['id'])
+
+		if len(properties) > 0:
+			props = '{ %s }' % (' '.join(properties))
+		else:
+			props = ''
+
+		# Handle image links by nesting the image inside a link
+		if 'href' in attrib:
+			href = attrib.get('href')
+			return ['[![%s](%s)%s](%s)' % (text, src, props, href)]
+
+		return ['![%s](%s)%s' % (text, src, props)]
 
 	def dump_object_fallback(self, tag, attrib, strings=None):
 		# dump object as verbatim block
@@ -122,15 +160,13 @@ class Dumper(TextDumper):
 		table += [rowline(rows[0])]
 		table.append(headsep)
 		table += [rowline(row) for row in rows[1:]]
-		return map(lambda line: line + "\n", table)
-
-	def dump_th(self, tag, attrib, strings):
-		strings = [s.replace('\n', '<br>').replace('|', '∣') for s in strings]
-		return [self._concat(strings)]
+		return [line + "\n" for line in table]
 
 	def dump_td(self, tag, attrib, strings):
-		strings = [s.replace('\n', '<br>').replace('|', '∣') for s in strings]
-		return [self._concat(strings)]
+		text = ''.join(strings) if strings else ''
+		return [escape_string(text.replace('\n', '<br>'), '|')]
+
+	dump_th = dump_td
 
 	def dump_line(self, tag, attrib, strings=None):
 		return '\n{}\n'.format('*' * 5)
