@@ -1,5 +1,5 @@
 
-# Copyright 2009-2022 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2009-2024 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -11,17 +11,19 @@ import logging
 import re
 
 import zim.datetimetz as datetime
-from zim.utils import natural_sorted
+from zim.base.naturalsort import natural_sorted
 
 from zim.notebook import Path
-from zim.actions import get_actions, RadioActionMethod, PRIMARY_MODIFIER_MASK
+from zim.actions import toggle_action, initialize_actiongroup, PRIMARY_MODIFIER_MASK
 from zim.signals import DelayedCallback, SIGNAL_AFTER, SignalHandler, ConnectorMixin
-from zim.config import ConfigManager
+from zim.config import ConfigManager, ConfigDefinition
+from zim.config.dicts import String
 from zim.plugins import PluginManager, ExtensionBase, extendable
 from zim.gui.widgets import \
 	Dialog, WindowSidePaneWidget, InputEntry, \
 	BrowserTreeView, SingleClickTreeView, ScrolledWindow, HPaned, StatusPage, \
-	encode_markup_text, decode_markup_text, widget_set_css
+	encode_markup_text, decode_markup_text, widget_set_css, \
+	uistate_property
 from zim.gui.actionextension import ActionExtensionBase, populate_toolbar_with_actions
 from zim.gui.clipboard import Clipboard
 
@@ -57,6 +59,30 @@ STATUS_COL = 11
 STATUS_ICON_NAME_COL = 12
 
 
+class TaskStatusUIState(ConfigDefinition):
+	'''Data type for selected task status list in uistate'''
+
+	def check(self, value):
+		# Valid data is sequence of integers for the TASK_STATUS_* constants
+		if isinstance(value, str):
+			try:
+				value = [int(s) for s in value.split(',')]
+			except:
+				raise ValueError
+		else:
+			if not (
+				isinstance(value, (list, tuple))
+				and all(isinstance(v, int) for v in value)
+			):
+				raise ValueError
+
+		if not all((v >= 0 and v <= 4) for v in value):
+			# In range of TASK_STATUS_OPEN to TASK_STATUS_TRANSMIGRATED
+			raise ValueError
+
+		return value
+
+
 class TaskListWidgetMixin(object):
 	'''Common functions between side-pane widget and dialog'''
 
@@ -69,13 +95,11 @@ class TaskListWidgetMixin(object):
 		SELECTION_WAITING: (_('Waiting'), WaitingTasks), # T: list with tasks (GTD style category)
 	}
 
+	taskselection_type = uistate_property('task_list', SELECTION_ALL, tuple(SELECTION_MAP.keys()))
+	status = uistate_property('task_status', [TASK_STATUS_OPEN], TaskStatusUIState)
+
 	def __init__(self, index, uistate, properties):
 		self.index = index
-		self.taskselection_type = SELECTION_ALL
-		self.taskselection = AllTasks.new_from_index(index)
-		self.status = [TASK_STATUS_OPEN]
-		self.label_tag_filter = (None, None, None)
-
 		self.tasklisttreeview = None
 		self.selection_list = None
 		self.tag_list = None
@@ -85,8 +109,16 @@ class TaskListWidgetMixin(object):
 		self.uistate.setdefault('sort_column', 0)
 		self.uistate.setdefault('sort_order', int(Gtk.SortType.DESCENDING))
 
+		self.taskselection = self.SELECTION_MAP[self.taskselection_type][1].new_from_index(index)
+		self.label_tag_filter = (None, None, None) # NOTE: Not taken from uistate since encoding&validation would be non-trivial
+
 		self.connectto(properties, 'changed', self.on_properties_changed)
 
+	def _init_selection_state(self):
+		# Ensure widgets reflect selection
+		state = self._get_selection_state()
+		self._set_selection_state(state)
+
 	def _get_selection_state(self):
 		return (self.taskselection_type, self.status[:], self.label_tag_filter)
 
@@ -259,6 +291,8 @@ class TaskListWidget(Gtk.VBox, TaskListWidgetMixin, WindowSidePaneWidget):
 		filter_entry = self._create_filter_entry()
 		self.pack_end(filter_entry, False, True, 0)
 
+		self._init_selection_state()
+
 	def set_embeded_closebutton(self, button):
 		if self._close_button:
 			self._header_hbox.remove(self._close_button)
@@ -277,6 +311,7 @@ class TaskListWindowExtension(ActionExtensionBase):
 	initialization.
 	'''
 
+	# TODO - should be generic base class for extending windows
 	# TODO - remove actions on teardown !!!
 
 	def __init__(self, plugin, window):
@@ -288,8 +323,10 @@ class TaskListWindowExtension(ActionExtensionBase):
 	def on_destroy(self, dialog):
 		self.destroy()
 
+
 from zim.config import value_is_coord
 
+
 @extendable(TaskListWindowExtension)
 class TaskListWindow(TaskListWidgetMixin, ConnectorMixin, Gtk.Window):
 
@@ -357,33 +394,65 @@ class TaskListWindow(TaskListWidgetMixin, ConnectorMixin, Gtk.Window):
 		pane2_vbox = Gtk.VBox()
 		self.hpane.pack2(pane2_vbox, resize=True)
 
-		swindow = self._create_tasklisttreeview(navigation, properties)
 		vbox.add(self.hpane)
+
+		self._search_bar = Gtk.SearchBar()
+		pane2_vbox.pack_start(self._search_bar, False, True, 0)
+		self._filter_entry = self._create_filter_entry()
+		self._filter_entry.set_size_request(400, -1) # FIXME, scale as percentage of window size / pane size ?
+		self._search_bar.add(self._filter_entry)
+		self._search_bar.connect_entry(self._filter_entry)
+		self._filter_entry.show_all()
+		self.connect('key-press-event', lambda o, e: self._search_bar.handle_event(e))
+		self._search_bar.connect('notify::search-mode-enabled', lambda *a: self.show_search(active=self._search_bar.get_search_mode()))
+			# make toggle button consistent when e.g. closing bar with <Escape> key binding
+
+		swindow = self._create_tasklisttreeview(navigation, properties)
 		pane2_vbox.pack_start(swindow, True, True, 0)
 
 		self.tasklisttreeview.view_columns['task'].set_min_width(400) # don't let this column get too small
 
-		self._filter_entry = self._create_filter_entry()
-		pane2_vbox.pack_start(self._filter_entry, False, True, 0)
-
 		self.hpane.pack1(self._create_selection_pane(properties, show_inbox_next, width=150), resize=False)
 
+		self._init_count_label()
+		self.tasklisttreeview.connect('view-changed', self._update_count_label)
+
 		if self._headerbar:
 			popover = self._create_menu()
 			button = self._create_menu_button(popover)
 			self._headerbar.pack_end(button)
+			self._headerbar.pack_end(self.show_search.create_icon_button())
+			self._headerbar.pack_end(self._count_label)
 		else:
+			assert self._toolbar is not None
 			self._update_toolbar()
 
+		self._init_selection_state()
+
+		# Initialze actions
+		initialize_actiongroup(self, 'win')
+
 		# Hack to add Ctrl-F keybinding for the filter
-		# in future should be handled by action / toolbar icon
-		# maybe move filter to top of window for consistent Gnome HiG look
+		# FIXME: would prefer Window base class to handle actions out of the box
 		group = Gtk.AccelGroup()
 		group.connect( # <Primary><F>
 			Gdk.unicode_to_keyval(ord('f')), PRIMARY_MODIFIER_MASK, Gtk.AccelFlags.VISIBLE,
-			lambda *a: self._filter_entry.grab_focus() )
+			lambda *a: self.show_search() )
 		self.add_accel_group(group)
 
+	def _init_count_label(self):
+		self._count_label = Gtk.Label()
+		self._count_label.set_margin_start(12)
+		self._count_label.set_margin_end(12)
+		#context = self._count_label.get_style_context()
+		#context.add_class(Gtk.STYLE_CLASS_SUBTITLE)
+		model = self.tasklisttreeview.get_model()
+		count = len(model) if model else 0
+		self._update_count_label(None, count)
+
+	def _update_count_label(self, view, count):
+		self._count_label.set_text(_('%s shown') % str(count)) # T: number of tasks shown in tasklist window
+
 	def _update_toolbar(self):
 		for item in self._toolbar.get_children():
 			self._toolbar.remove(item)
@@ -395,6 +464,14 @@ class TaskListWindow(TaskListWidgetMixin, ConnectorMixin, Gtk.Window):
 		space.set_expand(True)
 		self._toolbar.insert(space, -1)
 
+		item = Gtk.ToolItem()
+		item.add(self._count_label)
+		self._toolbar.insert(item, -1)
+
+		item = self.show_search.create_tool_button(connect_button=False)
+		item.set_action_name('win.show_search')
+		self._toolbar.insert(item, -1)
+
 		popover = self._create_menu()
 		button = self._create_menu_toolbutton(popover)
 		self._toolbar.insert(button, -1)
@@ -482,6 +559,11 @@ class TaskListWindow(TaskListWidgetMixin, ConnectorMixin, Gtk.Window):
 		except:
 			logger.exception('Exception in save_uistate')
 
+	@toggle_action(_('_Search'), '<Primary>F', verb_icon='edit-find-symbolic') # T: Menu item
+	def show_search(self, show):
+		self._search_bar.set_search_mode(show)
+
+
 
 class ListSelectionView(Gtk.ListBox):
 
@@ -526,9 +608,11 @@ class ListSelectionView(Gtk.ListBox):
 			if row.get_child()._zim_key == key:
 				self.select_row(row)
 				self.emit('row-activated', row)
-				return
+				return True
 		else:
-			raise AssertionError("Could not find key: %s" % key)
+			# NOTE: do not make this a fatal error, might be old uistate in place
+			logger.debug("Could not find selection key: %s" % key)
+			return False
 
 
 class LabelAndTagView(Gtk.ListBox):
@@ -677,13 +761,6 @@ class LabelAndTagView(Gtk.ListBox):
 				row.set_header(None)
 
 
-HIGH_COLOR = '#EF5151' # red (derived from Tango style guide - #EF2929)
-MEDIUM_COLOR = '#FCB956' # orange ("idem" - #FCAF3E)
-ALERT_COLOR = '#FCEB65' # yellow ("idem" - #FCE94F)
-# FIXME: should these be configurable ?
-
-COLORS = [None, ALERT_COLOR, MEDIUM_COLOR, HIGH_COLOR] # index 0..3
-
 _cal_days_to_work_days = [
 	# for each weekday 5 offsets used in algo below
 	# represent weekends in a 14 day period starting at given weekday
@@ -727,6 +804,16 @@ def days_to_str(days, use_workweek, weekday):
 
 class TaskListTreeView(BrowserTreeView):
 
+	# These default values are overwritten based on "styles.conf" configuration
+	HIGH_COLOR = '#ef2929' # red (derived from Tango style guide)
+	MEDIUM_COLOR = '#f57900' # orange
+	ALERT_COLOR = '#fce947' # yellow
+
+	PRIO_COLORS = [None, ALERT_COLOR, MEDIUM_COLOR, HIGH_COLOR] # index 0..3
+
+	TAG_TEXT_COLOR = '#ce5c00'
+	INACTIVE_TEXT_COLOR = 'darkgrey'
+
 	# define signals we want to use - (closure type, return type and arg types)
 	__gsignals__ = {
 		'view-changed': (GObject.SignalFlags.RUN_LAST, None, (object,)),
@@ -738,6 +825,14 @@ class TaskListTreeView(BrowserTreeView):
 		use_workweek=False,
 		sort_column=PRIO_COL, sort_order=Gtk.SortType.DESCENDING
 	):
+		self.text_style = ConfigManager.get_config_dict('style.conf')
+		self.text_style['TaskList Prio High'].define({'background': String(self.HIGH_COLOR)})
+		self.text_style['TaskList Prio Medium'].define({'background': String(self.MEDIUM_COLOR)})
+		self.text_style['TaskList Prio Alert'].define({'background': String(self.ALERT_COLOR)})
+		self.text_style['TaskList Inactive'].define({'foreground': String(self.INACTIVE_TEXT_COLOR)})
+		self.text_style.connect('changed', lambda o: self.on_text_style_changed())
+		self.on_text_style_changed()
+
 		self.real_model = Gtk.TreeStore(bool, bool, int, str, str, object, str, str, int, int, str, int, str)
 			# VIS_COL, ACT_COL, PRIO_COL, START_COL, DUE_COL, TAGS_COL, DESC_COL, PAGE_COL, TASKID_COL, PRIO_SORT_COL, PRIO_SORT_LABEL_COL, STATUS_COL, STATUS_ICON_NAME_COL
 		model = self.real_model.filter_new()
@@ -779,11 +874,11 @@ class TaskListTreeView(BrowserTreeView):
 		def render_prio(col, cell, model, i, data):
 			text = model.get_value(i, PRIO_SORT_LABEL_COL)
 			if not model.get_value(i, ACT_COL):
-				text = '<span color="darkgrey">%s</span>' % text
+				text = '<span color="%s">%s</span>' % (self.INACTIVE_TEXT_COLOR, text)
 				bg = None
 			else:
 				prio = model.get_value(i, PRIO_COL)
-				bg = COLORS[min(prio, 3)]
+				bg = self.PRIO_COLORS[min(prio, 3)]
 			cell.set_property('markup', text)
 			cell.set_property('cell-background', bg)
 
@@ -830,18 +925,18 @@ class TaskListTreeView(BrowserTreeView):
 				cell.set_property('text', '')
 			else:
 				if not model.get_value(i, ACT_COL):
-					date = '<span color="darkgrey">%s</span>' % date
+					date = '<span color="%s">%s</span>' % (self.INACTIVE_TEXT_COLOR, date)
 				cell.set_property('markup', date)
 				# TODO allow strftime here
 
 			if model_col == DUE_COL:
 				if date <= today:
-					color = HIGH_COLOR
+					color = self.HIGH_COLOR
 				elif date <= tomorrow:
-					color = MEDIUM_COLOR
+					color = self.MEDIUM_COLOR
 				elif date <= dayafter:
 					# "<=" because tomorrow and/or dayafter can be after the weekend
-					color = ALERT_COLOR
+					color = self.ALERT_COLOR
 				else:
 					color = None
 				cell.set_property('cell-background', color)
@@ -863,7 +958,7 @@ class TaskListTreeView(BrowserTreeView):
 			text = model.get_value(i, PAGE_COL)
 			text = encode_markup_text(text)
 			if not model.get_value(i, ACT_COL):
-				text = '<span color="darkgrey">%s</span>' % text
+				text = '<span color="%s">%s</span>' % (self.INACTIVE_TEXT_COLOR, text)
 			cell.set_property('markup', text)
 
 		cell_renderer = Gtk.CellRendererText()
@@ -883,6 +978,15 @@ class TaskListTreeView(BrowserTreeView):
 		self.connect('row_activated', self.__class__.do_row_activated)
 		self.connect('focus-in-event', self.__class__.do_focus_in_event)
 
+	def on_text_style_changed(self, *a):
+		self.HIGH_COLOR = self.text_style['TaskList Prio High']['background']
+		self.MEDIUM_COLOR = self.text_style['TaskList Prio Medium']['background']
+		self.ALERT_COLOR = self.text_style['TaskList Prio Alert']['background']
+		self.PRIO_COLORS = [None, self.ALERT_COLOR, self.MEDIUM_COLOR, self.HIGH_COLOR] # index 0..3
+
+		self.TAG_TEXT_COLOR = self.text_style['Tag tag'].get('foreground', self.TAG_TEXT_COLOR)
+		self.INACTIVE_TEXT_COLOR = self.text_style['TaskList Inactive']['foreground']
+
 	def get_view_column_visible(self, key):
 		assert key in self.view_columns
 		return self.view_columns[key].get_visible()
@@ -975,13 +1079,13 @@ class TaskListTreeView(BrowserTreeView):
 
 			# Format description
 			desc = _date_re.sub('', row['description'])
-			desc = re.sub('\s*!+\s*', ' ', desc) # get rid of exclamation marks
+			desc = re.sub(r'\s*!+\s*', ' ', desc) # get rid of exclamation marks
 			desc = encode_markup_text(desc)
 			if actionable:
-				desc = _tag_re.sub(r'<span color="#ce5c00">@\1</span>', desc) # highlight tags - same color as used in pageview
+				desc = _tag_re.sub(r'<span color="%s">@\1</span>' % self.TAG_TEXT_COLOR, desc) # highlight tags
 				desc = task_label_re.sub(r'<b>\1</b>', desc) # highlight labels
 			else:
-				desc = r'<span color="darkgrey">%s</span>' % desc
+				desc = '<span color="%s">%s</span>' % (self.INACTIVE_TEXT_COLOR, desc)
 
 			# Insert all columns
 			modelrow = [False, actionable, row['prio'], row['start'], row['due'], tags, desc, path.name, row['id'], prio_sort_int, prio_sort_label, status, status_icon_name]
@@ -1100,7 +1204,7 @@ class TaskListTreeView(BrowserTreeView):
 		text = self._get_raw_text(model[path])
 
 		pageview = self.opener.open_page(page)
-		pageview.find(text)
+		pageview.show_find(text)
 
 	def _get_raw_text(self, task):
 		id = task[TASKID_COL]
@@ -1197,7 +1301,7 @@ class TaskListTreeView(BrowserTreeView):
 
 <table class="tasklist">
 <tr><th>Prio</th><th>Task</th><th>Due</th><th>Start</th><th>Page</th></tr>
-''' % (HIGH_COLOR, MEDIUM_COLOR, ALERT_COLOR)
+''' % (self.HIGH_COLOR, self.MEDIUM_COLOR, self.ALERT_COLOR)
 
 		today = str(datetime.date.today())
 		tomorrow = str(datetime.date.today() + datetime.timedelta(days=1))
